@@ -1,5 +1,6 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2007-2012 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -52,18 +53,12 @@ int main (int argc, char *argv [])
         return -1;
     }
 
-    s = zmq_socket (ctx, ZMQ_SUB);
+    s = zmq_socket (ctx, ZMQ_PULL);
     if (!s) {
         printf ("error in zmq_socket: %s\n", zmq_strerror (errno));
         return -1;
     }
 
-    rc = zmq_setsockopt (s, ZMQ_SUBSCRIBE , "", 0);
-    if (rc != 0) {
-        printf ("error in zmq_setsockopt: %s\n", zmq_strerror (errno));
-        return -1;
-    }
-
     //  Add your socket options here.
     //  For example ZMQ_RATE, ZMQ_RECOVERY_IVL and ZMQ_MCAST_LOOP for PGM.
 
@@ -79,9 +74,9 @@ int main (int argc, char *argv [])
         return -1;
     }
 
-    rc = zmq_recv (s, &msg, 0);
-    if (rc != 0) {
-        printf ("error in zmq_recv: %s\n", zmq_strerror (errno));
+    rc = zmq_recvmsg (s, &msg, 0);
+    if (rc < 0) {
+        printf ("error in zmq_recvmsg: %s\n", zmq_strerror (errno));
         return -1;
     }
     if (zmq_msg_size (&msg) != message_size) {
@@ -92,9 +87,9 @@ int main (int argc, char *argv [])
     watch = zmq_stopwatch_start ();
 
     for (i = 0; i != message_count - 1; i++) {
-        rc = zmq_recv (s, &msg, 0);
-        if (rc != 0) {
-            printf ("error in zmq_recv: %s\n", zmq_strerror (errno));
+        rc = zmq_recvmsg (s, &msg, 0);
+        if (rc < 0) {
+            printf ("error in zmq_recvmsg: %s\n", zmq_strerror (errno));
             return -1;
         }
         if (zmq_msg_size (&msg) != message_size) {
