@@ -1,5 +1,6 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
+    Copyright (c) 2007-2009 iMatix Corporation
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -27,12 +28,15 @@
 #include <algorithm>
 
 #include "err.hpp"
-
-#include "../include/zmq.h"
+#include "msg.hpp"
+#include "i_decoder.hpp"
+#include "stdint.hpp"
 
 namespace zmq
 {
 
+    class i_msg_sink;
+
     //  Helper base class for decoders that know the amount of data to read
     //  in advance at any moment. Knowing the amount in advance is a property
     //  of the protocol used. 0MQ framing protocol is based size-prefixed
@@ -44,14 +48,14 @@ namespace zmq
     //  This class implements the state machine that parses the incoming buffer.
     //  Derived class should implement individual state machine actions.
 
-    template <typename T> class decoder_base_t
+    template <typename T> class decoder_base_t : public i_decoder
     {
     public:
 
         inline decoder_base_t (size_t bufsize_) :
+            next (NULL),
             read_pos (NULL),
             to_read (0),
-            next (NULL),
             bufsize (bufsize_)
         {
             buf = (unsigned char*) malloc (bufsize_);
@@ -139,6 +143,32 @@ namespace zmq
             }
         }
 
+        //  Returns true if the decoder has been fed all required data
+        //  but cannot proceed with the next decoding step.
+        //  False is returned if the decoder has encountered an error.
+        bool stalled ()
+        {
+            //  Check whether there was decoding error.
+            if (unlikely (!(static_cast <T*> (this)->next)))
+                return false;
+
+            while (!to_read) {
+                if (!(static_cast <T*> (this)->*next) ()) {
+                    if (unlikely (!(static_cast <T*> (this)->next)))
+                        return false;
+                    return true;
+                }
+            }
+
+            return false;
+        }
+
+        inline bool message_ready_size (size_t msg_sz)
+        {
+            zmq_assert (false);
+            return false;
+        }
+
     protected:
 
         //  Prototype of state machine action. Action should return false if
@@ -164,10 +194,18 @@ namespace zmq
 
     private:
 
-        unsigned char *read_pos;
-        size_t to_read;
+        //  Next step. If set to NULL, it means that associated data stream
+        //  is dead. Note that there can be still data in the process in such
+        //  case.
         step_t next;
 
+        //  Where to store the read data.
+        unsigned char *read_pos;
+
+        //  How much data to read before taking next step.
+        size_t to_read;
+
+        //  The duffer for data to decode.
         size_t bufsize;
         unsigned char *buf;
 
@@ -181,10 +219,11 @@ namespace zmq
     {
     public:
 
-        decoder_t (size_t bufsize_);
+        decoder_t (size_t bufsize_, int64_t maxmsgsize_);
         ~decoder_t ();
 
-        void set_inout (struct i_inout *destination_);
+        //  Set the receiver of decoded messages.
+        void set_msg_sink (i_msg_sink *msg_sink_);
 
     private:
 
@@ -193,9 +232,11 @@ namespace zmq
         bool flags_ready ();
         bool message_ready ();
 
-        struct i_inout *destination;
+        i_msg_sink *msg_sink;
         unsigned char tmpbuf [8];
-        ::zmq_msg_t in_progress;
+        msg_t in_progress;
+
+        int64_t maxmsgsize;
 
         decoder_t (const decoder_t&);
         void operator = (const decoder_t&);
