@@ -1,5 +1,6 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
+    Copyright (c) 2007-2009 iMatix Corporation
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -21,10 +22,9 @@
 #ifndef __ZMQ_KQUEUE_HPP_INCLUDED__
 #define __ZMQ_KQUEUE_HPP_INCLUDED__
 
-#include "platform.hpp"
-
-#if defined ZMQ_HAVE_FREEBSD || defined ZMQ_HAVE_OPENBSD ||\
-    defined ZMQ_HAVE_OSX || defined ZMQ_HAVE_NETBSD
+//  poller.hpp decides which polling mechanism to use.
+#include "poller.hpp"
+#if defined ZMQ_USE_KQUEUE
 
 #include <vector>
 
@@ -35,6 +35,8 @@
 namespace zmq
 {
 
+    struct i_poll_events;
+
     //  Implements socket polling mechanism using the BSD-specific
     //  kqueue interface.
 
@@ -48,7 +50,7 @@ namespace zmq
         ~kqueue_t ();
 
         //  "poller" concept.
-        handle_t add_fd (fd_t fd_, struct i_poll_events *events_);
+        handle_t add_fd (fd_t fd_, zmq::i_poll_events *events_);
         void rm_fd (handle_t handle_);
         void set_pollin (handle_t handle_);
         void reset_pollin (handle_t handle_);
@@ -79,7 +81,7 @@ namespace zmq
             fd_t fd;
             bool flag_pollin;
             bool flag_pollout;
-            i_poll_events *reactor;
+            zmq::i_poll_events *reactor;
         };
 
         //  List of retired event sources.
@@ -96,6 +98,8 @@ namespace zmq
         const kqueue_t &operator = (const kqueue_t&);
     };
 
+    typedef kqueue_t poller_t;
+
 }
 
 #endif
