@@ -1,5 +1,7 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
+    Copyright (c) 2007-2009 iMatix Corporation
+    Copyright (c) 2010-2011 Miru Limited
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -29,10 +31,10 @@
 #endif
 
 #include "pgm_receiver.hpp"
-#include "err.hpp"
+#include "session_base.hpp"
 #include "stdint.hpp"
 #include "wire.hpp"
-#include "i_inout.hpp"
+#include "err.hpp"
 
 zmq::pgm_receiver_t::pgm_receiver_t (class io_thread_t *parent_, 
       const options_t &options_) :
@@ -40,7 +42,7 @@ zmq::pgm_receiver_t::pgm_receiver_t (class io_thread_t *parent_,
     has_rx_timer (false),
     pgm_socket (true, options_),
     options (options_),
-    inout (NULL),
+    session (NULL),
     mru_decoder (NULL),
     pending_bytes (0)
 {
@@ -57,18 +59,22 @@ int zmq::pgm_receiver_t::init (bool udp_encapsulation_, const char *network_)
     return pgm_socket.init (udp_encapsulation_, network_);
 }
 
-void zmq::pgm_receiver_t::plug (io_thread_t *io_thread_, i_inout *inout_)
+void zmq::pgm_receiver_t::plug (io_thread_t *io_thread_,
+    session_base_t *session_)
 {
     //  Retrieve PGM fds and start polling.
-    int socket_fd;
-    int waiting_pipe_fd;
+    fd_t socket_fd = retired_fd;
+    fd_t waiting_pipe_fd = retired_fd;
     pgm_socket.get_receiver_fds (&socket_fd, &waiting_pipe_fd);
     socket_handle = add_fd (socket_fd);
     pipe_handle = add_fd (waiting_pipe_fd);
     set_pollin (pipe_handle);
     set_pollin (socket_handle);
 
-    inout = inout_;
+    session = session_;
+
+    //  If there are any subscriptions already queued in the session, drop them.
+    drop_subscriptions ();
 }
 
 void zmq::pgm_receiver_t::unplug ()
@@ -91,7 +97,7 @@ void zmq::pgm_receiver_t::unplug ()
     rm_fd (socket_handle);
     rm_fd (pipe_handle);
 
-    inout = NULL;
+    session = NULL;
 }
 
 void zmq::pgm_receiver_t::terminate ()
@@ -102,7 +108,7 @@ void zmq::pgm_receiver_t::terminate ()
 
 void zmq::pgm_receiver_t::activate_out ()
 {
-    zmq_assert (false);
+    drop_subscriptions ();
 }
 
 void zmq::pgm_receiver_t::activate_in ()
@@ -111,8 +117,15 @@ void zmq::pgm_receiver_t::activate_in ()
     //  processed the whole buffer but failed to write
     //  the last message into the pipe.
     if (pending_bytes == 0) {
-        if (mru_decoder != NULL)
+        if (mru_decoder != NULL) {
             mru_decoder->process_buffer (NULL, 0);
+            session->flush ();
+        }
+
+        //  Resume polling.
+        set_pollin (pipe_handle);
+        set_pollin (socket_handle);
+
         return;
     }
 
@@ -122,6 +135,7 @@ void zmq::pgm_receiver_t::activate_in ()
     //  Ask the decoder to process remaining data.
     size_t n = mru_decoder->process_buffer (pending_ptr, pending_bytes);
     pending_bytes -= n;
+    session->flush ();
 
     if (pending_bytes > 0)
         return;
@@ -139,7 +153,8 @@ void zmq::pgm_receiver_t::in_event ()
     unsigned char *data = NULL;
     const pgm_tsi_t *tsi = NULL;
 
-    zmq_assert (pending_bytes == 0);
+    if (pending_bytes > 0)
+        return;
 
     if (has_rx_timer) {
         cancel_timer (rx_timer_id);
@@ -215,9 +230,10 @@ void zmq::pgm_receiver_t::in_event ()
             it->second.joined = true;
 
             //  Create and connect decoder for the peer.
-            it->second.decoder = new (std::nothrow) decoder_t (0);
+            it->second.decoder = new (std::nothrow) decoder_t (0,
+                options.maxmsgsize);
             alloc_assert (it->second.decoder);
-            it->second.decoder->set_inout (inout);
+            it->second.decoder->set_msg_sink (session);
         }
 
         mru_decoder = it->second.decoder;
@@ -243,7 +259,7 @@ void zmq::pgm_receiver_t::in_event ()
     }
 
     //  Flush any messages decoder may have produced.
-    inout->flush ();
+    session->flush ();
 }
 
 void zmq::pgm_receiver_t::timer_event (int token)
@@ -255,5 +271,13 @@ void zmq::pgm_receiver_t::timer_event (int token)
     in_event ();
 }
 
+void zmq::pgm_receiver_t::drop_subscriptions ()
+{
+    msg_t msg;
+    msg.init ();
+    while (session->pull_msg (&msg))
+        msg.close ();
+}
+
 #endif
 
