@@ -1,5 +1,7 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
+    Copyright (c) 2007-2009 iMatix Corporation
+    Copyright (c) 2010-2011 Miru Limited
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -39,19 +41,22 @@
 namespace zmq
 {
 
+    class io_thread_t;
+    class session_base_t;
+
     class pgm_sender_t : public io_object_t, public i_engine
     {
 
     public:
 
-        pgm_sender_t (class io_thread_t *parent_, const options_t &options_);
+        pgm_sender_t (zmq::io_thread_t *parent_, const options_t &options_);
         ~pgm_sender_t ();
 
         int init (bool udp_encapsulation_, const char *network_);
 
         //  i_engine interface implementation.
-        void plug (class io_thread_t *io_thread_, struct i_inout *inout_);
-        void unplug ();
+        void plug (zmq::io_thread_t *io_thread_,
+            zmq::session_base_t *session_);
         void terminate ();
         void activate_in ();
         void activate_out ();
@@ -63,6 +68,9 @@ namespace zmq
 
     private:
 
+        //  Unplug the engine from the session.
+        void unplug ();
+
         //  TX and RX timeout timer ID's.
         enum {tx_timer_id = 0xa0, rx_timer_id = 0xa1};
 
