@@ -1,5 +1,7 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
+    Copyright (c) 2009-2011 250bpm s.r.o.
+    Copyright (c) 2007-2009 iMatix Corporation
+    Copyright (c) 2010-2011 Miru Limited
     Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
@@ -38,7 +40,7 @@
 #include "pgm_socket.hpp"
 #include "config.hpp"
 #include "err.hpp"
-#include "uuid.hpp"
+#include "random.hpp"
 #include "stdint.hpp"
 
 #ifndef MSG_ERRQUEUE
@@ -57,16 +59,14 @@ zmq::pgm_socket_t::pgm_socket_t (bool receiver_, const options_t &options_) :
 {
 }
 
-//  Create, bind and connect PGM socket.
+//  Resolve PGM socket address.
 //  network_ of the form <interface & multicast group decls>:<IP port>
 //  e.g. eth0;239.192.0.1:7500
 //       link-local;224.250.0.1,224.250.0.2;224.250.0.3:8000
 //       ;[fe80::1%en0]:7500
-int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
+int zmq::pgm_socket_t::init_address (const char *network_,
+    struct pgm_addrinfo_t **res, uint16_t *port_number)
 {
-    //  Can not open transport before destroying old one. 
-    zmq_assert (sock == NULL);
- 
     //  Parse port number, start from end for IPv6
     const char *port_delim = strrchr (network_, ':');
     if (!port_delim) {
@@ -74,7 +74,7 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
         return -1;
     }
 
-    uint16_t port_number = atoi (port_delim + 1);
+    *port_number = atoi (port_delim + 1);
   
     char network [256];
     if (port_delim - network_ >= (int) sizeof (network) - 1) {
@@ -83,45 +83,54 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
     }
     memset (network, '\0', sizeof (network));
     memcpy (network, network_, port_delim - network_);
-   
-    //  Validate socket options 
-    //  Data rate is in [B/s]. options.rate is in [kb/s].
-    if (options.rate <= 0) {
-        errno = EINVAL;
-        return -1;
-    }
-    //  Recovery interval [s] or [ms] - based on the user's call 
-    if ((options.recovery_ivl <= 0) && (options.recovery_ivl_msec <= 0)) {
-        errno = EINVAL;
-        return -1;
+
+    pgm_error_t *pgm_error = NULL;
+    struct pgm_addrinfo_t hints;
+
+    memset (&hints, 0, sizeof (hints));
+    hints.ai_family = AF_UNSPEC;
+    if (!pgm_getaddrinfo (network, NULL, res, &pgm_error)) {
+
+        //  Invalid parameters don't set pgm_error_t.
+        zmq_assert (pgm_error != NULL);
+        if (pgm_error->domain == PGM_ERROR_DOMAIN_IF &&
+
+              //  NB: cannot catch EAI_BADFLAGS.
+            ( pgm_error->code != PGM_ERROR_SERVICE &&
+              pgm_error->code != PGM_ERROR_SOCKTNOSUPPORT)) {
+
+            //  User, host, or network configuration or transient error.
+            pgm_error_free (pgm_error);
+            errno = EINVAL;
+            return -1;
+        }
+
+        //  Fatal OpenPGM internal error.
+        zmq_assert (false);
     }
+    return 0;
+}
+
+//  Create, bind and connect PGM socket.
+int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
+{
+    //  Can not open transport before destroying old one.
+    zmq_assert (sock == NULL);
+    zmq_assert (options.rate > 0);
 
     //  Zero counter used in msgrecv.
     nbytes_rec = 0;
     nbytes_processed = 0;
     pgm_msgv_processed = 0;
 
-    pgm_error_t *pgm_error = NULL;
-    struct pgm_addrinfo_t hints, *res = NULL;
+    uint16_t port_number;
+    struct pgm_addrinfo_t *res = NULL;
     sa_family_t sa_family;
 
-    memset (&hints, 0, sizeof (hints));
-    hints.ai_family = AF_UNSPEC;
-    if (!pgm_getaddrinfo (network, NULL, &res, &pgm_error)) {
+    pgm_error_t *pgm_error = NULL;
 
-        //  Invalid parameters don't set pgm_error_t.
-        zmq_assert (pgm_error != NULL);
-        if (pgm_error->domain == PGM_ERROR_DOMAIN_IF && (
-
-              //  NB: cannot catch EAI_BADFLAGS.
-              pgm_error->code != PGM_ERROR_SERVICE &&
-              pgm_error->code != PGM_ERROR_SOCKTNOSUPPORT))
-
-            //  User, host, or network configuration or transient error.
-            goto err_abort;
-
-        //  Fatal OpenPGM internal error.
-        zmq_assert (false);
+    if (init_address(network_, &res, &port_number) < 0) {
+        goto err_abort;
     }
 
     zmq_assert (res != NULL);
@@ -179,24 +188,24 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
     }
 
     {
-        const int rcvbuf = (int) options.rcvbuf,
-                  sndbuf = (int) options.sndbuf,
-                  max_tpdu = (int) pgm_max_tpdu;
-        if (rcvbuf) {
-            if (!pgm_setsockopt (sock, SOL_SOCKET, SO_RCVBUF, &rcvbuf,
-                  sizeof (rcvbuf)))
-                goto err_abort;
-        }
-        if (sndbuf) {
-            if (!pgm_setsockopt (sock, SOL_SOCKET, SO_SNDBUF, &sndbuf,
-                  sizeof (sndbuf)))
-                goto err_abort;
-        }
+		const int rcvbuf = (int) options.rcvbuf;
+		if (rcvbuf) {
+		    if (!pgm_setsockopt (sock, SOL_SOCKET, SO_RCVBUF, &rcvbuf,
+		          sizeof (rcvbuf)))
+		        goto err_abort;
+		}
 
-        //  Set maximum transport protocol data unit size (TPDU).
-        if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_MTU, &max_tpdu,
-              sizeof (max_tpdu)))
-            goto err_abort;
+		const int sndbuf = (int) options.sndbuf;
+		if (sndbuf) {
+		    if (!pgm_setsockopt (sock, SOL_SOCKET, SO_SNDBUF, &sndbuf,
+		          sizeof (sndbuf)))
+		        goto err_abort;
+		}
+
+		const int max_tpdu = (int) pgm_max_tpdu;
+		if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_MTU, &max_tpdu,
+		      sizeof (max_tpdu)))
+		    goto err_abort;
     }
 
     if (receiver) {
@@ -232,7 +241,7 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
             goto err_abort;
     } else {
         const int send_only        = 1,
-                  max_rte          = (int) ((options.rate * 1000) / 8),
+                  max_rte      = (int) ((options.rate * 1000) / 8),
                   txw_max_tpdu     = (int) pgm_max_tpdu,
                   txw_sqns         = compute_sqns (txw_max_tpdu),
                   ambient_spm      = pgm_secs (30),
@@ -266,20 +275,13 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
     addr.sa_port = port_number;
     addr.sa_addr.sport = DEFAULT_DATA_SOURCE_PORT;
 
-    if (options.identity.size () > 0) {
+    //  Create random GSI.
+    uint32_t buf [2];
+    buf [0] = generate_random ();
+    buf [1] = generate_random ();
+    if (!pgm_gsi_create_from_data (&addr.sa_addr.gsi, (uint8_t*) buf, 8))
+        goto err_abort;
 
-        //  Create gsi from identity.
-        if (!pgm_gsi_create_from_data (&addr.sa_addr.gsi,
-              options.identity.data (), options.identity.size ()))
-            goto err_abort;
-    } else {
-
-        //  Generate random gsi.
-        std::string gsi_base = uuid_t ().to_string ();
-        if (!pgm_gsi_create_from_string (&addr.sa_addr.gsi,
-              gsi_base.c_str (), -1))
-            goto err_abort;
-    }
 
     //  Bind a transport to the specified network devices.
     struct pgm_interface_req_t if_req;
@@ -324,24 +326,28 @@ int zmq::pgm_socket_t::init (bool udp_encapsulation_, const char *network_)
 
     //  Set IP level parameters.
     {
-        const int nonblocking      = 1,
-                  multicast_loop   = options.use_multicast_loop ? 1 : 0,
-                  multicast_hops   = 16,
+		// Multicast loopback disabled by default
+		const int multicast_loop = 0;
+		if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_MULTICAST_LOOP,
+		      &multicast_loop, sizeof (multicast_loop)))
+		    goto err_abort;
 
-                  //  Expedited Forwarding PHB for network elements, no ECN.
-                  dscp             = 0x2e << 2; 
+		const int multicast_hops = options.multicast_hops;
+		if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_MULTICAST_HOPS,
+		        &multicast_hops, sizeof (multicast_hops)))
+		    goto err_abort;
 
-        if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_MULTICAST_LOOP,
-                &multicast_loop, sizeof (multicast_loop)) ||
-            !pgm_setsockopt (sock, IPPROTO_PGM, PGM_MULTICAST_HOPS,
-                &multicast_hops, sizeof (multicast_hops)))
-            goto err_abort;
-        if (AF_INET6 != sa_family && !pgm_setsockopt (sock,
-              IPPROTO_PGM, PGM_TOS, &dscp, sizeof (dscp)))
-            goto err_abort;
-        if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_NOBLOCK,
-              &nonblocking, sizeof (nonblocking)))
-            goto err_abort;
+		//  Expedited Forwarding PHB for network elements, no ECN.
+		//  Ignore return value due to varied runtime support.
+		const int dscp = 0x2e << 2;
+		if (AF_INET6 != sa_family)
+		    pgm_setsockopt (sock, IPPROTO_PGM, PGM_TOS,
+			&dscp, sizeof (dscp));
+
+		const int nonblocking = 1;
+		if (!pgm_setsockopt (sock, IPPROTO_PGM, PGM_NOBLOCK,
+		      &nonblocking, sizeof (nonblocking)))
+		    goto err_abort;
     }
 
     //  Connect PGM transport to start state machine.
@@ -467,7 +473,7 @@ size_t zmq::pgm_socket_t::send (unsigned char *data_, size_t data_len_)
     //  We have to write all data as one packet.
     if (nbytes > 0) {
         zmq_assert (status == PGM_IO_STATUS_NORMAL);
-        zmq_assert ((ssize_t) nbytes == (ssize_t) data_len_);
+        zmq_assert (nbytes == data_len_);
     } else {
         zmq_assert (status == PGM_IO_STATUS_RATE_LIMITED ||
             status == PGM_IO_STATUS_WOULD_BLOCK);
@@ -681,20 +687,15 @@ void zmq::pgm_socket_t::process_upstream ()
 int zmq::pgm_socket_t::compute_sqns (int tpdu_)
 {
     //  Convert rate into B/ms.
-    uint64_t rate = ((uint64_t) options.rate) / 8;
-
-    //  Get recovery interval in milliseconds.
-    uint64_t interval = options.recovery_ivl_msec >= 0 ?
-        options.recovery_ivl_msec :
-        options.recovery_ivl * 1000;
+    uint64_t rate = uint64_t (options.rate) / 8;
         
     //  Compute the size of the buffer in bytes.
-    uint64_t size = interval * rate;
+    uint64_t size = uint64_t (options.recovery_ivl) * rate;
 
     //  Translate the size into number of packets.
     uint64_t sqns = size / tpdu_;
 
-    //  Buffer should be able to contain at least one packet.
+    //  Buffer should be able to hold at least one packet.
     if (sqns == 0)
         sqns = 1;
 
