@@ -1,6 +1,6 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2010-2011 250bpm s.r.o.
+    Copyright (c) 2010-2011 Other contributors as noted in the AUTHORS file
 
     This file is part of 0MQ.
 
@@ -30,7 +30,8 @@
     defined ZMQ_HAVE_HPUX || defined ZMQ_HAVE_AIX ||\
     defined ZMQ_HAVE_NETBSD
 #define ZMQ_SIGNALER_WAIT_BASED_ON_POLL
-#elif defined ZMQ_HAVE_WINDOWS || defined ZMQ_HAVE_OPENVMS
+#elif defined ZMQ_HAVE_WINDOWS || defined ZMQ_HAVE_OPENVMS ||\
+	defined ZMQ_HAVE_CYGWIN
 #define ZMQ_SIGNALER_WAIT_BASED_ON_SELECT
 #endif
 
@@ -57,16 +58,20 @@
 
 #include "signaler.hpp"
 #include "likely.hpp"
+#include "stdint.hpp"
+#include "config.hpp"
 #include "err.hpp"
 #include "fd.hpp"
 #include "ip.hpp"
 
+#if defined ZMQ_HAVE_EVENTFD
+#include <sys/eventfd.h>
+#endif
+
 #if defined ZMQ_HAVE_WINDOWS
 #include "windows.hpp"
 #else
 #include <unistd.h>
-#include <fcntl.h>
-#include <limits.h>
 #include <netinet/tcp.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -80,34 +85,25 @@ zmq::signaler_t::signaler_t ()
     errno_assert (rc == 0);
 
     //  Set both fds to non-blocking mode.
-#if defined ZMQ_HAVE_WINDOWS
-    unsigned long argp = 1;
-    rc = ioctlsocket (w, FIONBIO, &argp);
-    wsa_assert (rc != SOCKET_ERROR);
-    rc = ioctlsocket (r, FIONBIO, &argp);
-    wsa_assert (rc != SOCKET_ERROR);
-#else
-    int flags = fcntl (w, F_GETFL, 0);
-    errno_assert (flags >= 0);
-    rc = fcntl (w, F_SETFL, flags | O_NONBLOCK);
-    errno_assert (rc == 0);
-    flags = fcntl (r, F_GETFL, 0);
-    errno_assert (flags >= 0);
-    rc = fcntl (r, F_SETFL, flags | O_NONBLOCK);
-    errno_assert (rc == 0);
-#endif
+    unblock_socket (w);
+    unblock_socket (r);
 }
 
 zmq::signaler_t::~signaler_t ()
 {
-#if defined ZMQ_HAVE_WINDOWS
+#if defined ZMQ_HAVE_EVENTFD
+    int rc = close (r);
+    errno_assert (rc == 0);
+#elif defined ZMQ_HAVE_WINDOWS
     int rc = closesocket (w);
     wsa_assert (rc != SOCKET_ERROR);
     rc = closesocket (r);
     wsa_assert (rc != SOCKET_ERROR);
 #else
-    close (w);
-    close (r);
+    int rc = close (w);
+    errno_assert (rc == 0);
+    rc = close (r);
+    errno_assert (rc == 0);
 #endif
 }
 
@@ -118,7 +114,11 @@ zmq::fd_t zmq::signaler_t::get_fd ()
 
 void zmq::signaler_t::send ()
 {
-#if defined ZMQ_HAVE_WINDOWS
+#if defined ZMQ_HAVE_EVENTFD
+    const uint64_t inc = 1;
+    ssize_t sz = write (w, &inc, sizeof (inc));
+    errno_assert (sz == sizeof (inc));
+#elif defined ZMQ_HAVE_WINDOWS
     unsigned char dummy = 0;
     int nbytes = ::send (w, (char*) &dummy, sizeof (dummy), 0);
     wsa_assert (nbytes != SOCKET_ERROR);
@@ -144,7 +144,7 @@ int zmq::signaler_t::wait (int timeout_)
     pfd.events = POLLIN;
     int rc = poll (&pfd, 1, timeout_);
     if (unlikely (rc < 0)) {
-        zmq_assert (errno == EINTR);
+        errno_assert (errno == EINTR);
         return -1;
     }
     else if (unlikely (rc == 0)) {
@@ -173,7 +173,7 @@ int zmq::signaler_t::wait (int timeout_)
     int rc = select (r + 1, &fds, NULL, NULL,
         timeout_ >= 0 ? &timeout : NULL);
     if (unlikely (rc < 0)) {
-        zmq_assert (errno == EINTR);
+        errno_assert (errno == EINTR);
         return -1;
     }
 #endif
@@ -192,8 +192,24 @@ int zmq::signaler_t::wait (int timeout_)
 void zmq::signaler_t::recv ()
 {
     //  Attempt to read a signal.
+#if defined ZMQ_HAVE_EVENTFD
+    uint64_t dummy;
+    ssize_t sz = read (r, &dummy, sizeof (dummy));
+    errno_assert (sz == sizeof (dummy));
+
+    //  If we accidentally grabbed the next signal along with the current
+    //  one, return it back to the eventfd object.
+    if (unlikely (dummy == 2)) {
+        const uint64_t inc = 1;
+        ssize_t sz = write (w, &inc, sizeof (inc));
+        errno_assert (sz == sizeof (inc));
+        return;
+    }
+
+    zmq_assert (dummy == 1);
+#else
     unsigned char dummy;
-#ifdef ZMQ_HAVE_WINDOWS
+#if defined ZMQ_HAVE_WINDOWS
     int nbytes = ::recv (r, (char*) &dummy, sizeof (dummy), 0);
     wsa_assert (nbytes != SOCKET_ERROR);
 #else
@@ -202,11 +218,46 @@ void zmq::signaler_t::recv ()
 #endif
     zmq_assert (nbytes == sizeof (dummy));
     zmq_assert (dummy == 0);
+#endif
 }
 
 int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
 {
-#if defined ZMQ_HAVE_WINDOWS
+#if defined ZMQ_HAVE_EVENTFD
+
+    // Create eventfd object.
+    fd_t fd = eventfd (0, 0);
+    errno_assert (fd != -1);
+    *w_ = fd;
+    *r_ = fd;
+    return 0;
+
+#elif defined ZMQ_HAVE_WINDOWS
+    SECURITY_DESCRIPTOR sd = {0};
+    SECURITY_ATTRIBUTES sa = {0};
+
+    InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
+    SetSecurityDescriptorDacl(&sd, TRUE, 0, FALSE);
+
+    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
+    sa.lpSecurityDescriptor = &sd;
+
+    //  This function has to be in a system-wide critical section so that
+    //  two instances of the library don't accidentally create signaler
+    //  crossing the process boundary.
+    //  We'll use named event object to implement the critical section.
+    //  Note that if the event object already exists, the CreateEvent requests
+    //  EVENT_ALL_ACCESS access right. If this fails, we try to open
+    //  the event object asking for SYNCHRONIZE access only.
+    HANDLE sync = CreateEvent (&sa, FALSE, TRUE, TEXT ("Global\\zmq-signaler-port-sync"));
+    if (sync == NULL && GetLastError () == ERROR_ACCESS_DENIED)
+      sync = OpenEvent (SYNCHRONIZE | EVENT_MODIFY_STATE, FALSE, TEXT ("Global\\zmq-signaler-port-sync"));
+
+    win_assert (sync != NULL);
+
+    //  Enter the critical section.
+    DWORD dwrc = WaitForSingleObject (sync, INFINITE);
+    zmq_assert (dwrc == WAIT_OBJECT_0);
 
     //  Windows has no 'socketpair' function. CreatePipe is no good as pipe
     //  handles cannot be polled on. Here we create the socketpair by hand.
@@ -215,7 +266,7 @@ int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
 
     //  Create listening socket.
     SOCKET listener;
-    listener = socket (AF_INET, SOCK_STREAM, 0);
+    listener = open_socket (AF_INET, SOCK_STREAM, 0);
     wsa_assert (listener != INVALID_SOCKET);
 
     //  Set SO_REUSEADDR and TCP_NODELAY on listening socket.
@@ -233,15 +284,10 @@ int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
     memset (&addr, 0, sizeof (addr));
     addr.sin_family = AF_INET;
     addr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
-    addr.sin_port = 0;
+    addr.sin_port = htons (signaler_port);
     rc = bind (listener, (const struct sockaddr*) &addr, sizeof (addr));
     wsa_assert (rc != SOCKET_ERROR);
 
-    //  Retrieve local port listener is bound to (into addr).
-    int addrlen = sizeof (addr);
-    rc = getsockname (listener, (struct sockaddr*) &addr, &addrlen);
-    wsa_assert (rc != SOCKET_ERROR);
-
     //  Listen for incomming connections.
     rc = listen (listener, 1);
     wsa_assert (rc != SOCKET_ERROR);
@@ -250,23 +296,39 @@ int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
     *w_ = WSASocket (AF_INET, SOCK_STREAM, 0, NULL, 0,  0);
     wsa_assert (*w_ != INVALID_SOCKET);
 
+    //  On Windows, preventing sockets to be inherited by child processes.
+    BOOL brc = SetHandleInformation ((HANDLE) *w_, HANDLE_FLAG_INHERIT, 0);
+    win_assert (brc);
+
     //  Set TCP_NODELAY on writer socket.
     rc = setsockopt (*w_, IPPROTO_TCP, TCP_NODELAY,
         (char *)&tcp_nodelay, sizeof (tcp_nodelay));
     wsa_assert (rc != SOCKET_ERROR);
 
     //  Connect writer to the listener.
-    rc = connect (*w_, (sockaddr *) &addr, sizeof (addr));
+    rc = connect (*w_, (struct sockaddr*) &addr, sizeof (addr));
     wsa_assert (rc != SOCKET_ERROR);
 
     //  Accept connection from writer.
     *r_ = accept (listener, NULL, NULL);
     wsa_assert (*r_ != INVALID_SOCKET);
 
+    //  On Windows, preventing sockets to be inherited by child processes.
+    brc = SetHandleInformation ((HANDLE) *r_, HANDLE_FLAG_INHERIT, 0);
+    win_assert (brc);
+
     //  We don't need the listening socket anymore. Close it.
     rc = closesocket (listener);
     wsa_assert (rc != SOCKET_ERROR);
 
+    //  Exit the critical section.
+    brc = SetEvent (sync);
+    win_assert (brc != 0);
+
+    // Release the kernel object
+    brc = CloseHandle (sync);
+    win_assert (brc != 0);
+
     return 0;
 
 #elif defined ZMQ_HAVE_OPENVMS
@@ -277,13 +339,13 @@ int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
     //
     //  The bug will be fixed in V5.6 ECO4 and beyond.  In the meantime, we'll
     //  create the socket pair manually.
-    sockaddr_in lcladdr;
+    struct sockaddr_in lcladdr;
     memset (&lcladdr, 0, sizeof (lcladdr));
     lcladdr.sin_family = AF_INET;
     lcladdr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
     lcladdr.sin_port = 0;
 
-    int listener = socket (AF_INET, SOCK_STREAM, 0);
+    int listener = open_socket (AF_INET, SOCK_STREAM, 0);
     errno_assert (listener != -1);
 
     int on = 1;
@@ -304,7 +366,7 @@ int zmq::signaler_t::make_fdpair (fd_t *r_, fd_t *w_)
     rc = listen (listener, 1);
     errno_assert (rc != -1);
 
-    *w_ = socket (AF_INET, SOCK_STREAM, 0);
+    *w_ = open_socket (AF_INET, SOCK_STREAM, 0);
     errno_assert (*w_ != -1);
 
     rc = setsockopt (*w_, IPPROTO_TCP, TCP_NODELAY, &on, sizeof (on));
